/*
This class establishes the TCP/IP Connection with the CapTap, 
receives the commands and triggers the appropriate action

*/

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using UnityEngine.UI;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CapTapConnector : MonoBehaviour
{
    //Filename for debug-logging purposes
    private string fileName;
    private string mappedButton;

    public Knock recentKnock;
    public HandPosition recentLeftHand;

    private int xMax = 400;
    private int yMax = 400;


    public CapTapConnector()
    {
        recentLeftHand = new HandPosition(-1, -1, -1);
        //Logging path
        fileName = "/logging-" + DateTime.Now.Day + "-" + DateTime.Now.Month + "-" + DateTime.Now.Year + "-" + DateTime.Now.Hour + "-" + DateTime.Now.Minute + "-" + DateTime.Now.Second + ".dat";
    }


    public void startClient()
    {
        //string hostName = "pczotaccaptap";
        string hostName = "localhost";
        int port = 1234;

        //Connect to CapTap
        try
        {
            //Establish the remote endpoint for the socket.
            IPHostEntry ipHostInfo = Dns.GetHostEntry(hostName);
            IPAddress ipAddress = ipHostInfo.AddressList[0];
            IPEndPoint remoteEP = new IPEndPoint(ipAddress, port);

            //Create a TCP/IP socket.
            Socket sender = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            //Connect the socket to the remote endpoint. Catch any errors. 
            Debug.Log("Trying to connect to " + hostName + ":" + port);
            try
            {
                sender.Connect(remoteEP);

                Debug.Log("Socket connected to " + sender.RemoteEndPoint.ToString());

                //TODO: get available Ports from Table 

                //Init table
                string setPorts = "[SETPORTS,/dev/ttyUSB0,/dev/ttyUSB1,/dev/ttyUSB2]\n";
                sender.Send(Encoding.UTF8.GetBytes(setPorts));

                //Set the threshold value 
                string thresholdCommand = "[SET_THRESHOLD,20]\n";
                sender.Send(Encoding.UTF8.GetBytes(thresholdCommand));

                //Set the erode value
                string erodeCommand = "[SET_ERODE,7]\n";
                sender.Send(Encoding.UTF8.GetBytes(erodeCommand));

                //Set the sound threshold
                string soundThresholdCommand = "[SET_AUDIOTHRESH," + 750 + "]\n";
                sender.Send(Encoding.UTF8.GetBytes(soundThresholdCommand));

                //Set the sound timeout
                string soundTimeoutCommand = "[SET_AUDIOTIMEOUT,300]\n";
                sender.Send(Encoding.UTF8.GetBytes(soundTimeoutCommand));

                //turn off sending images
                string imgOff = "[EN_SEND_IMG,false]\n";
                sender.Send(Encoding.UTF8.GetBytes(imgOff));

                //Start table
                string startCommand = "[START]\n";
                sender.Send(Encoding.UTF8.GetBytes(startCommand));

                //Command the server to send us the max-values
                string maxCommand = "[GET_MAX]\n";
                sender.Send(Encoding.UTF8.GetBytes(maxCommand));

                byte[] buffer = new byte[512];
                //byte[] imgBuf = null;
                //int imgRead = 0;
                //int state = 0; // waiting for new message
                Debug.Log("Start Reading");

                while (true)
                {

                    // read next message
                    int read = sender.Receive(buffer);
                    if (read > 0)
                    {

                        // try to parse msg
                        // get byte values for comparison
                        byte paran1 = Convert.ToByte('[');
                        byte paran2 = Convert.ToByte(']');

                        // search buffer for both bytes
                        int start = -1;
                        int end = -1;

                        bool searchStart = true;
                        List<BufferPositions> positionList = new List<BufferPositions>();

                        //Search for all position markers in the buffer
                        for (int i = 0; i < read; i++)
                        {
                            if (searchStart && buffer[i] == paran1)
                            {
                                start = i;
                                searchStart = false;
                            }
                            else if (!searchStart && buffer[i] == paran2)
                            {
                                end = i;
                                searchStart = true;
                                //Save new BufferPosition-Couple in the List
                                positionList.Add(new BufferPositions(start, end));
                            }
                        }

                        //for each valid message execute relevant action
                        foreach (BufferPositions position in positionList)
                        {
                            String msg = new string(Encoding.UTF8.GetChars(buffer), position.getStart(), position.getEnd() - position.getStart() + 1);

                            //Debug Logging Purposes
                            //writeData(msg);

                            // parse msg
                            String[] parts = msg.Substring(1, msg.Length - 2).Split(',');

                            if (parts.Length == 3 && parts[0].Equals("MAX"))
                            {
                                setMaxValues(Convert.ToInt32(parts[1]), Convert.ToInt32(parts[2]));
                            }

                            //Parse the KNOCK-Message
                            if (parts[0].Equals("KNOCK"))
                            {
                                //Debug.Log(msg);
                                recentKnock = new Knock(Convert.ToDouble(parts[1]));
                                //As soon a knock occured, check the last received handPosition

                            }


                            //Parse the POS-Message
                            else if (parts.Length == 7 && parts[0].Equals("POS"))
                            {
                                //Debug.Log(msg);
                                recentLeftHand = new HandPosition(Convert.ToInt32(parts[1]), Convert.ToInt32(parts[2]), Convert.ToInt32(parts[3]));
                                //recentRightHand = new HandPosition(Convert.ToInt32(parts[4]), Convert.ToInt32(parts[5]), Convert.ToInt32(parts[6]));						             
                            }
                        }
                    }

                    // Wait 20ms 
                    System.Threading.Thread.Sleep(20);
                }
            }
            catch (ArgumentNullException ane)
            {
                Debug.LogError("ArgumentNullException :" + ane.ToString());
            }
            catch (SocketException se)
            {
                Debug.LogError("SocketException :" + se.ToString());
            }
            catch (Exception e)
            {
                Debug.LogError("Unexpected exception : " + e.ToString());
            }

            finally
            {
                if (sender != null)
                    sender.Close();
            }

        }
        catch (Exception e)
        {
            Debug.LogError("An Exception occured");
            Debug.LogError(e.ToString());
        }
    }

    /*
	 * Set the max-Values - Not in use
	 * These values are not really relevant right now because the actual position values still differ from this MaxValues
	 * The actual values
	 */

    public void setMaxValues(int xMax, int yMax)
    {
        this.xMax = xMax;
        this.yMax = yMax;
    }
}